#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This script is licensed under GNU GPL version 2.0 or above
# (c) 2023 Antonio J. Delgado
# List IMAP messages

import sys
import os
import logging
import click
import click_config_file
from logging.handlers import SysLogHandler
import imaplib
import re
import json
from operator import itemgetter, attrgetter

class Mailbox:
    def __init__(self, name, count):
        self.name = name
        self.count = count
    
    def __str__(self):
        return f"{self.name} {self.count}"

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
            sort_keys=True, indent=4)

    def __repr__(self):
        return repr((self.name, self.count))

class list_imap_messages:

    def __init__(self, debug_level, log_file, **kwargs):
        ''' Initial function called when object is created '''
        self.config = dict()
        self.config['debug_level'] = debug_level
        if log_file is None:
            log_file = os.path.join(os.environ.get('HOME', os.environ.get('USERPROFILE', os.getcwd())), 'log', 'list_imap_messages.log')
        self.config['log_file'] = log_file
        self.config.update(kwargs)
        self._init_log()
        if self.config['ssl']:
            self._connect_ssl()
        else:
            self._connect()
        self._login()
        #self._select_mailbox()
        self._list_mailboxes()

    def _list_mailboxes(self):
        self._log.debug(f"Listing mailboxes in {self.config['mailbox']}")
        result = self.imap.list()
        if result[0] == 'NO':
            self._log.error(f"Error listing mailboxes in {self.config['mailbox']}: {result[1]}")
            self.imap.close()
            self.imap.logout()
            sys.exit(1)
        else:
            self._log.debug(f"Result: {result}")
        mailboxes = list()
        for raw_mailbox in result[1]:
            self._log.debug(raw_mailbox)
            mailbox_list = raw_mailbox.decode().split(' "/" ')
            result = self._select_mailbox(mailbox_list[1])
            mailboxes.append(
                Mailbox(
                    mailbox_list[1].replace('"', ''),
                    int(result[1][0].decode())
                    )
                )
        print(json.dumps(sorted(mailboxes, key=lambda mailbox: mailbox.count, reverse=True), indent=2, default=vars))
        return result

    def _select_mailbox(self, mailbox):
        try:
            self._log.debug(f"Selecting mailbox '{self.config['username']}@{self.config['server']}:{self.config['port']}/{mailbox}'")
            result=self.imap.select(mailbox, True)
        except imaplib.IMAP4.error as error:
            self._log.error(f"Error selecting mailbox '{self.config['username']}@{self.config['server']}:{self.config['port']}/{mailbox}': {error}")
            self.imap.close()
            self.imap.logout()
            sys.exit(1)
        if result[0] == 'NO':
            self._log.error(f"Error selecting mailbox '{self.config['username']}@{self.config['server']}:{self.config['port']}/{mailbox}': {result[1]}")
            self.imap.close()
            self.imap.logout()
            sys.exit(1)
        else:
            self._log.debug(f"Result: {result}")
        return result

    def _login(self):
        try:
            self._log.debug(f"Login as '{self.config['username']}@{self.config['server']}:{self.config['port']}'")
            self.imap.login(self.config['username'], self.config['password'])
        #except imaplib.IMAP4.error as error:
        except Exception as error:
            self._log.error(f"Error login as '{self.config['username']}@{self.config['server']}:{self.config['port']}': {error}")
            self.imap.logout()
            sys.exit(1)

    def _connect_ssl(self):
        try:
            self._log.debug(f"Connecting to '{self.config['server']}:{self.config['port']}/SSL'")
            self.imap=imaplib.IMAP4_SSL(self.config['server'], self.config['port'])
        except Exception as error:
            self._log.error(f"Error connecting to '{self.config['server']}:{self.config['port']}': {error}")
            sys.exit(1)

    def _connect(self):
        try:
            self._log.debug(f"Connecting to '{self.config['server']}:{self.config['port']}/noSSL'")
            self.imap=imaplib.IMAP4(self.config['server'], self.config['port'])
        except Exception as error:
            self._log.error(f"Error connecting to '{self.config['server']}:{self.config['port']}': {error}")
            sys.exit(1)


    def _init_log(self):
        ''' Initialize log object '''
        self._log = logging.getLogger("list_imap_messages")
        self._log.setLevel(logging.DEBUG)

        sysloghandler = SysLogHandler()
        sysloghandler.setLevel(logging.DEBUG)
        self._log.addHandler(sysloghandler)

        streamhandler = logging.StreamHandler(sys.stdout)
        streamhandler.setLevel(logging.getLevelName(self.config.get("debug_level", 'INFO')))
        self._log.addHandler(streamhandler)

        if 'log_file' in self.config:
            log_file = self.config['log_file']
        else:
            home_folder = os.environ.get('HOME', os.environ.get('USERPROFILE', ''))
            log_folder = os.path.join(home_folder, "log")
            log_file = os.path.join(log_folder, "list_imap_messages.log")

        if not os.path.exists(os.path.dirname(log_file)):
            os.mkdir(os.path.dirname(log_file))

        filehandler = logging.handlers.RotatingFileHandler(log_file, maxBytes=102400000)
        # create formatter
        formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
        filehandler.setFormatter(formatter)
        filehandler.setLevel(logging.DEBUG)
        self._log.addHandler(filehandler)
        return True

@click.command()
@click.option("--debug-level", "-d", default="INFO",
    type=click.Choice(
        ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG", "NOTSET"],
        case_sensitive=False,
    ), help='Set the debug level for the standard output.')
@click.option('--log-file', '-l', help="File to store all debug messages.")
@click.option("--ssl","-s", is_flag=True, default=True, help="Use SSL connection usually port 993.")
@click.option("--server","-S", default='localhost', help="Server full name or IP.")
@click.option("--port","-p", default=993, help="Server port to connect.")
@click.option("--username","-u", required=True, help="User name to login.")
@click.option("--password","-w", required=True, help="User password to login.")
@click.option("--mailbox","-m", default='INBOX', help="Mailbox to start examination.")
@click_config_file.configuration_option()
def __main__(debug_level, log_file, **kwargs):
    return list_imap_messages(debug_level, log_file, **kwargs)

if __name__ == "__main__":
    __main__()

